from pygame import *
import unittest
import random
from lawn import Cat,Lawn

class CatState:
    default_poo_timer = 25
    default_lives = 9 # groan...
    
    def __init__(self):
        self.states=("init",
                     "spawning",
                     "scared",
                     "walking",
                     "running",
                     "crapping",
                     "escaping")
        self.state = "init"
        self.cat = Cat(0,0)
        self.state_rate = 100
        self.state_elapse = 0
        self.poo_timer = self.default_poo_timer
        self.lives = self.default_lives
        
    def _setState(self, state):
        if not state in self.states:
            raise NameError, 'invalid state set'
        self.state = state
        
    def _spawnCoords(self,lawn):   
        on_x_edge = random.randint(0, 1) == 0
        on_min_edge = random.randint(0, 1) == 0
        
        startx=0 - self.cat.rect.width
        starty=0 - self.cat.rect.height
        
        if not on_x_edge:
            if not on_min_edge:
                startx = lawn.rect.width
            starty = random.randint(0, lawn.rect.height)
        else:
            if not on_min_edge:
                starty = lawn.rect.height
            startx = random.randint(0, lawn.rect.width)
        
        return startx,starty     
    
    def spawn(self,lawn):
        if lawn.catOnLawn():
            raise NameError, 'CatState can only control one cat'
            
        startx,starty = self._spawnCoords(lawn)
        self.cat.topleft=(startx,starty)
        
        lawn.items["cat"] = self.cat
        self._setState("spawning")
        lawn.setCatDest()
    
    def after(self, lawn, millisecs):
        self.state_elapse = self.state_elapse + millisecs
        """        
        if lawn.killsCat():
            self._setState("escaping")
            self.state_elapse = 0
            self.lives = 0
            self.cat.dest = self._spawnCoords(lawn)
        """        
     
        if lawn.scaresCat():
            if self.lives == 0:
                self._setState("escaping")
                
            if self.state == "spawning":
                self._setState("scared")
                self.state_elapse = 0
                self.lives = self.lives - 1
                if self.lives == 0:
                    self._setState("escaping")
                    self.cat.dest = self._spawnCoords(lawn)
                
            elif self.state == "walking" or self.state == "running" or self.state ==  "crapping":
                self._setState("escaping")
                self.state_elapse = 0
                self.lives = self.lives - 1
                if self.lives == 0:
                    self.cat.dest = self._spawnCoords(lawn)
                else:
                    lawn.setDesperateCatDest()
            
        if self.state_elapse < self.state_rate:
            return
        
        if self.state == "spawning":
            self.state_elapse = 0
            self.cat.moveTowardsDest()
            if lawn.rect.contains(self.cat.rect):
                self._setState("walking")
                
        elif self.state == "walking" or self.state == "running":
            self.state_elapse = 0
            
            if self.state == "walking":
                self.cat.moveTowardsDest()
            else:
                self.cat.moveTowardsDest(2)
                
            if self.cat.atDest():
                self._setState("crapping")
         
        elif self.state == "crapping":
            self.state_elapse = 0
            self.poo_timer = self.poo_timer - 1
            if self.poo_timer <= 0 :
                self.cat.crap(lawn)
                self._setState("escaping")
                self.cat.dest = self._spawnCoords(lawn)
                    
        elif self.state == "scared":
            self.state_elapse = 0
            self.cat.moveAwayFromDest(2)
            
            if not lawn.rect.contains(self.cat.rect):
                del lawn.items["cat"]
                self.spawn(lawn)
                    
        elif self.state == "escaping":
            self.state_elapse = 0
            self.cat.moveTowardsDest(2)
            
            if self.cat.atDest():            
                if not lawn.rect.contains(self.cat.rect):
                    del lawn.items["cat"]
                    self._setState("init")
                    self.poo_timer = self.default_poo_timer
                    self.lives = self.default_lives
                else:
                    lawn.setCatDest()
                    self._setState("running")
                    if self.lives < 3:
                        lawn.setDesperateCatDest()
                    else:
                        lawn.setCatDest()
                    
        else:
            raise NameError, 'not finished yet!'
                    

class CatStateTest(unittest.TestCase):        
    def testSafeSpawnCat(self):
        lawn = Lawn()
        catState = CatState()
        #move scary items from edge of lawn
        lawn.items["mower"].rect.center = lawn.rect.center        
        lawn.items["gardener"].rect.center = lawn.rect.center
        
        catState.spawn(lawn)
        
        while catState.state == "spawning":
            catState.after(lawn,10)
        
        self.assertTrue(catState.state == "walking")
        self.assertTrue(lawn.rect.contains(catState.cat.rect))
        self.assertTrue(lawn.rect.contains(lawn.items["cat"]))  # sanity
            
    def testScaredSpawningCatRespawns(self):
        lawn = Lawn()
        catState = CatState()        
        
        catState.spawn(lawn)
        catState.cat.dest=(0,400)
        while catState.state == "spawning":
            catState.after(lawn,10)
            
        self.assertTrue(catState.state == "scared")
        
        while catState.state == "scared":
            catState.after(lawn,10)
            
        self.assertTrue(catState.state == "spawning")            
        
        x,y = catState.cat.rect.topleft
        self.assertTrue(x != 0 or y != 400) # should be random new posn
    
    def testCalmCatWalks(self):
        lawn = Lawn()
        catState = CatState()
        #move scary items from edge of lawn
        lawn.items["mower"].rect.center = lawn.rect.center        
        lawn.items["gardener"].rect.center = lawn.rect.center
        
        catState.spawn(lawn)
        
        while catState.state == "spawning":
            catState.after(lawn,10)
            
        self.assertTrue(catState.state == "walking") 
        
    def testScaredCatRuns(self):
        lawn = Lawn()
        catState = CatState()
        #move scary items from edge of lawn
        lawn.items["mower"].rect.center = lawn.rect.center        
        lawn.items["gardener"].rect.center = lawn.rect.center
        
        catState.spawn(lawn)
        
        while catState.state == "spawning":
            catState.after(lawn,10)
                    
        lawn.items["mower"].rect.center = catState.cat.dest       
        lawn.items["gardener"].rect.center = catState.cat.dest
        
        while catState.state == "walking":
            catState.after(lawn,10)
        
        self.assertTrue(catState.state == "escaping")
                    
        while catState.state == "escaping":
            catState.after(lawn,10)
            
        self.assertTrue(catState.state == "running")
        
    def testScaredCatCalmAtDest(self):
        lawn = Lawn()
        catState = CatState()
        #move scary items from edge of lawn
        lawn.items["mower"].rect.center = lawn.rect.center        
        lawn.items["gardener"].rect.center = lawn.rect.center
        
        catState.spawn(lawn)
        
        while catState.state == "spawning":
            catState.after(lawn,10)
                    
        lawn.items["mower"].rect.center = catState.cat.dest       
        lawn.items["gardener"].rect.center = catState.cat.dest
        
        while catState.state == "walking":
            catState.after(lawn,10)            
        
        while catState.state == "escaping":
            catState.after(lawn,10)
        
        del lawn.items["mower"]        
        del lawn.items["gardener"]
        
        while catState.state == "running":
            catState.after(lawn,10)
            
        posx,posy=catState.cat.rect.topleft
        destx,desty= catState.cat.dest
        
        self.assertEqual(posx,destx)
        self.assertEqual(posy,desty)
        
        self.assertTrue(catState.state == "crapping")
        
    def testCatCrapsAtDest(self):
        lawn = Lawn()
        catState = CatState()
        
        catState.spawn(lawn)
        
        del lawn.items["mower"]        
        del lawn.items["gardener"]
        
        while catState.state == "spawning":
            catState.after(lawn,10)                    
        
        while catState.state == "walking":
            catState.after(lawn,10)
        
        self.assertTrue(catState.state == "crapping")
        
    def testCatCrapsAfterPeristalsis(self):
        lawn = Lawn()
        catState = CatState()
        
        catState.spawn(lawn)
        
        del lawn.items["mower"]        
        del lawn.items["gardener"]
        
        while catState.state == "spawning":
            catState.after(lawn,10)                    
        
        while catState.state == "walking":
            catState.after(lawn,10)         
        
        peristalsis_cache = catState.poo_timer 
        while catState.state == "crapping":
            catState.after(lawn,100)
            self.assertTrue(catState.poo_timer == peristalsis_cache - 1)
            peristalsis_cache = catState.poo_timer
        
        self.assertTrue(catState.poo_timer == 0)
        self.assertEqual(len(lawn.items["crap"]), 1)
        
    def testCatScaredMidCrapCrapsQuickerAtNextDest(self):
        lawn = Lawn()
        catState = CatState()        
        catState.spawn(lawn)
        
        while catState.state != "crapping":
            catState.after(lawn,10)
        
        peristalsis_cache = catState.poo_timer 
        catState.after(lawn,200)  # have a bit of a crap    
        lawn.items["mower"].rect.topleft = catState.cat.rect.bottomright       
        lawn.items["gardener"].rect.topright = catState.cat.rect.bottomleft
        
        catState.after(lawn,10) # get scared
        while catState.state != "crapping":
            catState.after(lawn,10)
        
        self.assertTrue(peristalsis_cache > catState.poo_timer)
        
    def testCatEscapesAfterCrap(self):
        lawn = Lawn()
        catState = CatState()
        
        catState.spawn(lawn)
        
        del lawn.items["mower"]        
        del lawn.items["gardener"]
        
        while catState.state == "spawning":
            catState.after(lawn,10)                    
        
        while catState.state == "walking":
            catState.after(lawn,10)         
        
        while catState.state == "crapping":
            catState.after(lawn,10)
        
        while catState.state == "escaping":
            catState.after(lawn,10)
        
        self.assertFalse(lawn.catOnLawn())
        
    def testCatEscapesAfterMaxScareCount(self):
        lawn = Lawn()
        catState = CatState()
        
        catState.spawn(lawn)
        
        #move scary items from edge of lawn
        lawn.items["mower"].rect.center = lawn.rect.center        
        lawn.items["gardener"].rect.center = lawn.rect.center
   
        while catState.state == "spawning":
            catState.after(lawn,10)

        lawn.items["mower"].rect.center = catState.cat.dest       
        lawn.items["gardener"].rect.center = catState.cat.dest
                    
        while catState.state == "walking":
            catState.after(lawn,10)        
        
        lives = 10
        while catState.state == "escaping" or catState.state == "running":
            lawn.items["mower"].rect.center = catState.cat.dest       
            lawn.items["gardener"].rect.center = catState.cat.dest
            lives = catState.lives 
            catState.after(lawn,10) 
            
        self.assertTrue(catState.state == "init")
        self.assertTrue(lives == 0)
        self.assertFalse(lawn.catOnLawn())
        
    def testCatPrefersToCrapOnLongGrass(self):
        lawn = Lawn()
        catState = CatState()
        
        lanes_left = lawn.rect.width / lawn.items["mower"].rect.width
        deltay = -1
        
        while lanes_left > 1:             
            while lawn.items["mower"].crossing_lanes(lawn):
                lawn.items["mower"].rect.move_ip(1,0)
            
            lawn.items["mower"].mow(lawn)
            while lawn.items["mower"].mowing_lane(lawn):
                lawn.items["mower"].rect.move_ip(0,deltay)
                lawn.items["mower"].mow(lawn)
                
            deltay = -deltay               
            lanes_left = lanes_left - 1
        
        while lawn.items["mower"].crossing_lanes(lawn):
            lawn.items["mower"].rect.move_ip(1,0)
            
        catState.spawn(lawn)        
        while catState.state != "crapping":
            catState.after(lawn,10)
            
        x,y = catState.cat.dest
        self.assertTrue(x > lawn.rect.width - lawn.items["mower"].rect.width)
        
    def testCatCrapsOnShortGrassIfDesperate(self):        
        lawn = Lawn()
        catState = CatState()
        
        lanes_left = lawn.rect.width / lawn.items["mower"].rect.width
        deltay = -1
        
        while lanes_left > 1:             
            while lawn.items["mower"].crossing_lanes(lawn):
                lawn.items["mower"].rect.move_ip(1,0)
            
            lawn.items["mower"].mow(lawn)
            while lawn.items["mower"].mowing_lane(lawn):
                lawn.items["mower"].rect.move_ip(0,deltay)
                lawn.items["mower"].mow(lawn)
                
            deltay = -deltay               
            lanes_left = lanes_left - 1
        
        while lawn.items["mower"].crossing_lanes(lawn):
            lawn.items["mower"].rect.move_ip(1,0)
            
        lawn.items["mower"].mow(lawn)
        total_y = 0
        while lawn.items["mower"].mowing_lane(lawn) and total_y < (lawn.rect.height - 60):
            lawn.items["mower"].rect.move_ip(0,deltay)
            lawn.items["mower"].mow(lawn)
            total_y = total_y + 1 
        
        catState.spawn(lawn)        
        while catState.state != "crapping":
            catState.after(lawn,10)
        
        x,y = catState.cat.dest
        self.assertTrue(x < lawn.rect.width - lawn.items["mower"].rect.width)
    """                
    def testMaxScareCountReachedIfCatCollides(self):
        lawn = Lawn()
        catState = CatState()
        
        catState.spawn(lawn)
        
        #move scary items from edge of lawn
        lawn.items["mower"].rect.center = lawn.rect.center        
        lawn.items["gardener"].rect.center = lawn.rect.center
        
        
        while catState.state == "spawning":
            catState.after(lawn,10)

        lawn.items["mower"].rect.center = catState.cat.rect.center       
        lawn.items["gardener"].rect.center = catState.cat.rect.center
        catState.after(lawn,10)
                
        self.assertTrue(catState.state == "escaping")
        self.assertTrue(catState.lives == 0)
    """        
        
    def testCatEscapeDestOnGardenEdge(self):
        lawn = Lawn()
        catState = CatState()
        
        catState.spawn(lawn)
        
        del lawn.items["mower"]        
        del lawn.items["gardener"]
        
        while catState.state == "spawning":
            catState.after(lawn,10)                    
        
        while catState.state == "walking":
            catState.after(lawn,10)         
        
        while catState.state == "crapping":
            catState.after(lawn,10)
            
        x,y = catState.cat.dest
        width,height=lawn.rect.width,lawn.rect.height
        self.assertTrue(  ( (x==-catState.cat.rect.width or x==width) and y>=-catState.cat.rect.height and y<=height) \
                        or ((y==-catState.cat.rect.height or y==height) and x>=-catState.cat.rect.width and x<=width) )
    
    def testCatRunsWhenEscaping(self):#crap test
        lawn = Lawn()
        catState = CatState()
        
        catState.spawn(lawn)
        
        del lawn.items["mower"]        
        del lawn.items["gardener"]
        
        while catState.state == "spawning":
            catState.after(lawn,10)                    
        
        while catState.state == "walking":
            catState.after(lawn,10)         
        
        while catState.state == "crapping":
            catState.after(lawn,10)
        
        self.assertTrue(catState.state == "escaping")
    """        
    def testCatFindsAltEscapePathWhenScaredDuringEscape(self):
        lawn = Lawn()
        catState = CatState()       
                
        catState.spawn(lawn)
        
        while catState.state != "escaping":
            catState.after(lawn,10)
        
        oldx,oldy=catState.cat.dest
        lawn.items["mower"].rect.center = catState.cat.dest       
        lawn.items["gardener"].rect.center = catState.cat.dest
    
        while catState.state == "escaping":
            catState.after(lawn,10)
            
        newx,newy=catState.cat.dest
        self.assertTrue(newx != oldx and newy != oldy)
    """