import unittest

class Anim:
    def __init__(self,frame_times, looping):
        self.frame_index_times = frame_times
        self.looping = looping
        self.elapsed = 0
        self.total = 0
        for frame, time in frame_times:
            self.total = self.total + time
        self.next = 0   # should assert frame_times are valid really
        
    def after(self, millisecs):
        self.elapsed = self.elapsed + millisecs
        
        if self.elapsed >= self.total:
            if not self.looping:
                #self.next = -1
                return
            else:
                self.elapsed = self.elapsed % self.total
                
        total = 0
        for frame, time in self.frame_index_times:
            total = total + time
            if total > self.elapsed:
                self.next = frame
                return
        raise NameError, 'couldn\'t find anim frame for elapsed time'        

class AnimTest(unittest.TestCase):
    frame_times = ((0, 500),
                (1, 450),
                (2, 400))
    
    def testNonLoopingAnimSequence(self):          
        anim = Anim(self.frame_times, False)        
        self.assertEqual(0, anim.next)
        
        anim.after(100)
        self.assertEqual(0, anim.next)
        
        anim.after(401)
        self.assertEqual(1, anim.next)
        
        anim.after(451)
        self.assertEqual(2, anim.next)
        
        anim.after(401)
        self.assertEqual(-1, anim.next)
    
    def testLoopingAnimSequence(self):        
        anim = Anim(self.frame_times, True)        
        self.assertEqual(0, anim.next)
        
        anim.after(501)
        self.assertEqual(1, anim.next)
        
        anim.after(451)
        self.assertEqual(2, anim.next)
        
        anim.after(401)
        self.assertEqual(0, anim.next) 