from pygame import *
import unittest
import random

class Mower:
    def __init__(self, xpos, ypos):
        self.rect = Rect(xpos,ypos,30,50)
        self.crappy = False
        
    def mow(self, lawn):
        if len(lawn.mowed_rect) == 0:
            mowed_rect = Rect(self.rect)
            lawn.mowed_rect.append(mowed_rect)
        else:
            last_lane=lawn.mowed_rect[-1]
            if last_lane.height != lawn.rect.height:
                last_lane.union_ip(self.rect)
            else:
                mowed_rect = Rect(self.rect)
                lawn.mowed_rect.append(mowed_rect)
    
    def crossing_lanes(self, lawn):#ie should cross lanes
        if len(lawn.mowed_rect) == 0:
            return False
        
        lanes = lawn.rect.width / self.rect.width
        if len(lawn.mowed_rect) == lanes:
            return False 
        
        last_lane=lawn.mowed_rect[-1]     
        return last_lane.height == lawn.rect.height and \
                self.rect.left < last_lane.right            
    
    def mowing_lane(self, lawn):#ie should mow lane
        if len(lawn.mowed_rect) == 0:
            return True
        
        last_lane=lawn.mowed_rect[-1]
        if last_lane.height < lawn.rect.height:
            return True
        else: 
            return False
        
    def coveredInCrap(self, lawn):
        for crap in lawn.items["crap"]:
            if self.rect.colliderect(crap):
                self.crappy = True
                return self.crappy        
        return self.crappy
        
class Gardener:
    def __init__(self, xpos, ypos):
        self.rect = Rect(xpos,ypos,18,33)
        self.alive = True   # death is a peristent state
        
    def dead(self,lawn):
        """
        no idea WHAT I was thinking here
        if len(lawn.mowed_rect) != 0:
            for lanes in lawn.mowed_rect:
                if self.rect.colliderect(lanes):
                    self.alive = False
                    return not self.alive
        return not self.alive
        """
        # bit of a last min hack here to make the mower match the sprite size
        sprite_rect = lawn.items["mower"].rect.inflate(-10,0)
        self.alive = (sprite_rect.colliderect(self.rect) == False)
        return not self.alive
    
    def tidy(self, lawn):
        tidied = False
        for crap in lawn.items["crap"]:
            if self.rect.colliderect(crap):
                lawn.items["crap"].remove(crap)
                tidied = True
        return tidied
        

class Cat:
    def __init__(self, xpos, ypos):
        self.rect = Rect(xpos,ypos,20,40)
        self.dest = (xpos, ypos)
        
    def crap(self, lawn):
        crap_rect = Rect(0,0,5,10)
        crap_rect.center = self.rect.center
        lawn.items["crap"].append(crap_rect)
        
    def moveTowardsDest(self,speed=None):
        destx,desty = self.dest
        dir_x = destx - self.rect.left
        dir_y = desty - self.rect.top
        
        vel=0
        if speed==None or dir_x == 1 or dir_x == -1 or dir_y == 1 or dir_y == -1:
            vel=1
        else:
            vel=speed
        
        deltax, deltay = 0,0
        
        if(dir_x < 0):
            deltax = -1 * vel
        elif(dir_x > 0):
            deltax = 1 * vel       
        
        if(dir_y < 0):
            deltay = -1 * vel
        elif(dir_y > 0):
            deltay = 1 * vel
            
        self.rect.move_ip(deltax, deltay)
        
    def moveAwayFromDest(self,speed=None):
        destx,desty = self.dest
        dir_x = destx - self.rect.left
        dir_y = desty - self.rect.top
        
        vel=0
        if speed==None or dir_x == 1 or dir_x == -1 or dir_y == 1 or dir_y == -1:
            vel=1
        else:
            vel=speed
        
        deltax, deltay = 0,0
        
        if(dir_x < 0):
            deltax = -1 * vel
        elif(dir_x > 0):
            deltax = 1 * vel       
        
        if(dir_y < 0):
            deltay = -1 * vel
        elif(dir_y > 0):
            deltay = 1 * vel
            
        self.rect.move_ip(-deltax, -deltay)
        
    def atDest(self):
        x,y = self.dest
        return (x == self.rect.left and y == self.rect.top)
        
class Lawn:
    def __init__(self):
        self.rect = Rect(0,0,400,400)
        self.items = {}
        self.items["mower"] = Mower(0,350)
        self.items["gardener"] = Gardener(0,0)
        self.items["gardener"].rect.center = self.rect.center
        self.items["crap"] = []
        self.mowed_rect = []
        
    def catOnLawn(self):
        return "cat" in self.items
    
    def killsCat(self):
        if not self.catOnLawn():
            raise NameError, 'reallyScaresCat called when cat not spawned from hell'
        
        #purely for testing
        if not "mower" in self.items and not "gardener" in self.items:
            return False         
        
        return self.items["cat"].rect.colliderect(self.items["mower"].rect) or \
                self.items["cat"].rect.colliderect(self.items["gardener"].rect)
    
    def scaresCat(self):
        if not self.catOnLawn():
            raise NameError, 'scaresCat called when cat not spawned from hell'
        
        #purely for testing
        if not "mower" in self.items and not "gardener" in self.items:
            return False 
        
        fearRange=15
        mowerRange=self.items["mower"].rect.inflate(fearRange,fearRange)
        gardenerRange=self.items["gardener"].rect.inflate(fearRange,fearRange)
        
        return self.items["cat"].rect.colliderect(mowerRange) or \
                self.items["cat"].rect.colliderect(gardenerRange)
    
    def setDesperateCatDest(self):
        if not self.catOnLawn():
            raise NameError, 'setDesperateCatDest called when cat not spawned from hell'
        
        available_width = self.rect.width - self.items["cat"].rect.width
        available_height = self.rect.height - self.items["cat"].rect.height
        destx = random.randint(0, available_width)
        desty = random.randint(0, available_height)
        self.items["cat"].dest=(destx,desty)
    
    def setCatDest(self):
        if not self.catOnLawn():
            raise NameError, 'setCatDest called when cat not spawned from hell'
        
        lane_width = self.items["mower"].rect.width
        available_width = self.rect.width - self.items["cat"].rect.width
        available_height = self.rect.height - self.items["cat"].rect.height
        
        x,y,top=0,0,0
        
        for lane in self.mowed_rect:
            x = x + lane.width
            y = lane.height
            top = lane.top
            
        if x > available_width and y > available_height:
            self.setDesperateCatDest()
            return
        
        destx=0
        if x == 0: # ie. len(self.mowed_rect) == 0
            destx = random.randint(0, available_width)
        elif y > available_height:
            destx = random.randint(x, available_width)
        else:
            destx = random.randint(x - lane_width, available_width)
            
        desty = 0        
        if y < self.rect.height and destx < x:
            if top != 0:
                desty = random.randint(0, available_height - y)
            else:
                desty = random.randint(y,available_height)
        else:                
            desty = random.randint(0,available_height)
            
        self.items["cat"].dest=(destx,desty)
            
#test for success

class LawnTest(unittest.TestCase):
    expectedMowerRect=Rect(0,0,50,50)
    expectedMowerStartPosn=(0,350)
    
    expectedGardenerRect=Rect(0,0,10,30)
    expectedGardenerStartPosn=(195,185)
    
    expectedCatRect=Rect(0,0,20,40)
    expectedCatStartPosn=(195,10)
    catPosnNearMower=(57,360)
    catPosnNearGardener=(195,174)
    
    expectedCrapRect=Rect(0,0,5,10)
    expectedCrapStartPosn=(203,25)
    
    expectedLawnRect=Rect(0,0,400,400)
    #expectedStartingMowedRect=Rect(0,399,0,0) 
    expectedMowedLane=Rect(0,0,50,400)
    
    def testPosnAccess(self):
        lawn = Lawn()
        mowerRect = lawn.items["mower"].rect
        self.assertEqual(mowerRect.width, self.expectedMowerRect.width)
        self.assertEqual(mowerRect.height, self.expectedMowerRect.height)
        x,y = self.expectedMowerStartPosn
        self.assertEqual(mowerRect.left, x)
        self.assertEqual(mowerRect.top, y)
        
        gardenerRect = lawn.items["gardener"].rect
        self.assertEqual(gardenerRect.width, self.expectedGardenerRect.width)
        self.assertEqual(gardenerRect.height, self.expectedGardenerRect.height)
        x,y = self.expectedGardenerStartPosn
        self.assertEqual(gardenerRect.left, x)
        self.assertEqual(gardenerRect.top, y)
        
    def testCatOnLawn(self):
        lawn = Lawn()
        x,y = self.expectedCatStartPosn
        lawn.items["cat"] = Cat(x,y)
        self.assertTrue(lawn.catOnLawn())
        catRect = lawn.items["cat"].rect
        self.assertEqual(catRect.width, self.expectedCatRect.width)
        self.assertEqual(catRect.height, self.expectedCatRect.height)
        self.assertEqual(catRect.left, x)
        self.assertEqual(catRect.top, y)
        
    def testCrapOnLawn(self):
        lawn = Lawn()
        x,y = self.expectedCatStartPosn
        lawn.items["cat"] = Cat(x,y)
        lawn.items["cat"].crap(lawn)
        self.assertTrue(len(lawn.items["crap"]) != 0)
        crapRect = lawn.items["crap"][0]        
        self.assertEqual(crapRect.width, self.expectedCrapRect.width)
        self.assertEqual(crapRect.height, self.expectedCrapRect.height)
        x,y = self.expectedCrapStartPosn
        self.assertEqual(crapRect.left, x)
        self.assertEqual(crapRect.top, y)
        
    def testSelectCatDest(self):        
        lawn = Lawn()
        x,y = self.expectedCatStartPosn
        lawn.items["cat"] = Cat(x,y)
        lawn.setCatDest()
        x,y = lawn.items["cat"].dest
        
        while(lawn.items["cat"].atDest() == False):
            lawn.items["cat"].moveTowardsDest()                
        
        catRect = lawn.items["cat"].rect
        self.assertEqual(catRect.left, x)
        self.assertEqual(catRect.top, y)        
        
    def testCatScaredOfMower(self):
        lawn = Lawn()
        x,y = self.expectedCatStartPosn
        lawn.items["cat"] = Cat(x,y)
        self.assertFalse(lawn.scaresCat())
        
        x,y = self.catPosnNearMower
        lawn.items["cat"].dest =(x,y)
        
        while(lawn.items["cat"].atDest() == False):
            lawn.items["cat"].moveTowardsDest()
            
        self.assertTrue(lawn.scaresCat())
        
    def testCatScaredOfGardener(self):
        lawn = Lawn()
        x,y = self.expectedCatStartPosn
        lawn.items["cat"] = Cat(x,y)
        self.assertFalse(lawn.scaresCat())
        
        x,y = self.catPosnNearGardener
        lawn.items["cat"].dest =(x,y)
        
        while(lawn.items["cat"].atDest() == False):
            lawn.items["cat"].moveTowardsDest()
            
        self.assertTrue(lawn.scaresCat())
        
    def testMowedArea(self):   
        lawn = Lawn()
        self.assertEqual(self.expectedLawnRect, lawn.rect)
        self.assertEqual(0, len(lawn.mowed_rect))
        lawn.items["mower"].mow(lawn)
        self.assertEqual(lawn.items["mower"].rect, lawn.mowed_rect[0])
        
    def testAllMowed(self):
        lawn = Lawn()
        self.assertEqual(lawn.rect.width % lawn.items["mower"].rect.width, 0)
        lanes_left = lawn.rect.width / lawn.items["mower"].rect.width
        deltay = -1
        
        while lanes_left > 0:             
            while lawn.items["mower"].crossing_lanes(lawn):
                lawn.items["mower"].rect.move_ip(1,0)
            
            lawn.items["mower"].mow(lawn)
            while lawn.items["mower"].mowing_lane(lawn):
                lawn.items["mower"].rect.move_ip(0,deltay)
                lawn.items["mower"].mow(lawn)
                
            deltay = -deltay               
            lanes_left = lanes_left - 1 
        
        mowed_rect=Rect(0,lawn.rect.height,0,0)
        for lane in lawn.mowed_rect:
            mowed_rect = mowed_rect.union(lane)
        
        self.assertEqual(lawn.rect, mowed_rect)
        
    def testMowedCrap(self):
        lawn = Lawn()
        x,y = self.expectedCatStartPosn
        lawn.items["cat"] = Cat(x,y)
        lawn.items["cat"].crap(lawn)
        lawn.items["mower"].rect.topleft=self.expectedCatStartPosn
        
#        self.assertFalse(lawn.items["mower"].coveredInCrap(lawn))
        lawn.items["mower"].mow(lawn)        
        self.assertTrue(lawn.items["mower"].coveredInCrap(lawn))
        
#    def testMowedCat(self):
        
    def testMowedGardener(self):
        lawn = Lawn()
        lawn.items["mower"].rect.topleft=self.expectedGardenerStartPosn
        self.assertFalse(lawn.items["gardener"].dead(lawn))
        lawn.items["mower"].mow(lawn)
        self.assertTrue(lawn.items["gardener"].dead(lawn))
        
    def testMowedLane(self):
        lawn = Lawn()
        
        lawn.items["mower"].mow(lawn)        
        while lawn.items["mower"].mowing_lane(lawn):
            lawn.items["mower"].rect.move_ip(0,-1)
            lawn.items["mower"].mow(lawn)
        
        self.assertEqual(lawn.items["mower"].rect.top, 0)
        self.assertEqual(self.expectedMowedLane, lawn.mowed_rect[0]) 
        
    def testMowerMovedToNextLane(self):
        lawn = Lawn()
                
        while lawn.items["mower"].mowing_lane(lawn):
            lawn.items["mower"].mow(lawn)
            lawn.items["mower"].rect.move_ip(0,-1)
            
        while lawn.items["mower"].crossing_lanes(lawn):
            lawn.items["mower"].rect.move_ip(1,0)
        
        self.assertEqual(lawn.items["mower"].rect.left, 50)
        
    def testGardenerPickedUpCrap(self):
        lawn = Lawn()
        x,y = self.expectedCatStartPosn
        lawn.items["cat"] = Cat(x,y)
        lawn.items["cat"].crap(lawn)
        lawn.items["gardener"].rect.topleft = self.expectedCatStartPosn
        lawn.items["gardener"].tidy(lawn)
        self.assertEqual(len(lawn.items["crap"]),0)
            
            
# test for failure


#test for sanity  