'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

import data
from pygame import *
from lawn import *
from cat_state import *
from goml_anim import *


def update_mower(lawn, mower_deltay, in_lane):             
    complete=False
    if lawn.items["mower"].crossing_lanes(lawn):
        lawn.items["mower"].rect.move_ip(1,0)
        in_lane = False
        if not lawn.items["mower"].crossing_lanes(lawn):
            lawn.items["mower"].mow(lawn)
        
    elif lawn.items["mower"].mowing_lane(lawn):
        if not in_lane:
            in_lane = True
            mower_deltay = -mower_deltay
                                
        lawn.items["mower"].mow(lawn)
        lawn.items["mower"].rect.move_ip(0,mower_deltay)
    else:
        complete = True
        
    return mower_deltay, in_lane, complete

def main():
    mower_deltay = -1
    in_lane = True
    mower_crapped = False
    game_over = True
    success = False
    first_run = True # evil, but I want to drink beer 
    mower_frame_count = 12
    
    running = True
    
    lawn = Lawn()
    init()
    screen = display.set_mode((400,400))
    mouse.set_visible(False)
    display.toggle_fullscreen()
    display.set_caption('cat crap defender')
    
    scorefont = font.Font(None, 20)
    myClock = time.Clock()
    screen.fill((0,0,0))
    
    dark_green = Color('0x006600')
    green = Color('green')
    red = Color('red')
    black = Color('black')
    brown = Color('brown')
    white = Color('white')
    blue = Color('blue')
    
    catState = CatState()
    catState.spawn(lawn)
    
    lawn_graphic = Surface((lawn.rect.width,lawn.rect.height))
    mower_graphic = Surface((lawn.items["mower"].rect.width,lawn.items["mower"].rect.height))
    #cat_graphic = Surface((catState.cat.rect.width,catState.cat.rect.height)) 
    gardener_graphic = Surface((lawn.items["gardener"].rect.width,lawn.items["gardener"].rect.height))
    
    #mower_graphic.fill(red)
    #cat_graphic.fill(black)
    gardener_graphic.fill(blue)
    
    no_of_cats = 1
    incx, incy, incmag, incmax = 0,0,1,2
    tidy_flag = False
    
    mow_frames = []
    
    mow_frames.append(image.load(data.filepath('mower_down_one.gif')))
    mow_frames.append(image.load(data.filepath('mower_down_two.gif')))
    mow_frames.append(image.load(data.filepath('mower_up_one.gif')))
    mow_frames.append(image.load(data.filepath('mower_up_two.gif')))
    mow_frames.append(image.load(data.filepath('mower_down_crap.gif')))
    mower_down_crap_frame = len(mow_frames) - 1
    mow_frames.append(image.load(data.filepath('mower_up_crap.gif')))
    mower_up_crap_frame = len(mow_frames) - 1
    
    for frame in mow_frames:
        frame.set_colorkey(white)
    
    mower_up = Anim(    ((2,800),(3,800)),   True)
    mower_down = Anim(    ((0,800),(1,800)),   True)
    
    crap_graphic = image.load(data.filepath('poo.gif'))
    crap_graphic.set_colorkey(white)
    
    gdnr_frames = []
    
    gdnr_frames.append(image.load(data.filepath('gardener_down_one.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_down_two.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_up_one.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_up_two.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_clean_down_one.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_clean_down_two.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_clean_up_one.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_clean_up_two.gif')))
    gdnr_frames.append(image.load(data.filepath('gardener_dead.gif')))
    
    for frame in gdnr_frames:
        frame.set_colorkey(white)
        
    gardener_up = Anim(    ((2,800),(3,800)),   True)
    gardener_down = Anim(    ((0,800),(1,800)),   True)
    gardener_clean_up = Anim(    ((6,800),(7,800)),   True)
    gardener_clean_down = Anim(    ((4,800),(5,800)),   True)
    gardener_dead = Anim(    ((0,400),(8,400)),   True)
    
    cat_frames = []
    
    cat_frames.append(image.load(data.filepath('cat_down_one.gif')))
    cat_frames.append(image.load(data.filepath('cat_down_two.gif')))
    cat_frames.append(image.load(data.filepath('cat_down_three.gif')))
    cat_frames.append(image.load(data.filepath('cat_up_one.gif')))
    cat_frames.append(image.load(data.filepath('cat_up_two.gif')))
    cat_frames.append(image.load(data.filepath('cat_up_three.gif')))
    cat_frames.append(image.load(data.filepath('cat_crap_one.gif')))
    cat_frames.append(image.load(data.filepath('cat_crap_two.gif')))
    cat_frames.append(image.load(data.filepath('cat_crap_three.gif')))
    cat_frames.append(image.load(data.filepath('cat_scared.gif')))
    
    for frame in cat_frames:
        frame.set_colorkey(white)
    
    cat_up_walk = Anim(    ((3,800),(4,800),(4,800)),   True)
    cat_up_run = Anim(    ((3,400),(4,400),(4,400)),   True)
    cat_down_walk = Anim(    ((0,800),(1,800),(2,800)),   True)
    cat_down_run = Anim(    ((0,400),(1,400),(2,400)),   True)    
    cat_crap = Anim(((6,800),(7,800),(8,800)),   False)
    cat_scared = 9
    
    cat_frame = 0
    crap_sound = mixer.Sound(data.filepath('poo.ogg'))
    crapping = False
    scared_sound = mixer.Sound(data.filepath('scared_cat.ogg'))
    scared = lawn.scaresCat()
    dead_sound = mixer.Sound(data.filepath('dead.ogg'))
    dead_mower = mixer.Sound(data.filepath('mower_crapped.ogg'))
    mower_sound = mixer.Sound(data.filepath('mowing.ogg'))
    mower_sound_length = mower_sound.get_length() * 100
    mixer.Sound.play(mower_sound)
    
    title_screen = image.load(data.filepath('title.gif'))
    title_screen.set_colorkey(white)   
    
    while running:
        
        for the_event in event.get():
            if the_event.type == QUIT:
                running = False
            
            elif the_event.type == KEYDOWN:
                if the_event.key == K_DOWN:
                    incy = incy + incmag          
                elif the_event.key == K_UP:
                    incy = incy - incmag
                elif the_event.key == K_RIGHT:
                    incx = incx + incmag
                elif the_event.key == K_LEFT:
                    incx = incx - incmag
                                           
                elif the_event.key == K_LCTRL:
                    tidy_flag = True
                    
                elif the_event.key == K_ESCAPE or the_event.key == K_q: 
                    running = False
                    
                elif the_event.key == K_r:
                    first_run = False
                    lawn = Lawn()
                    catState = CatState()
                    catState.spawn(lawn)
                    mower_deltay = -1
                    in_lane = True
                    mower_crapped = False
                    game_over = False
                    mower_frame_count = 12
                    no_of_cats = 1
                    incx, incy, incmag, incmax = 0,0,1,2
                    tidy_flag = False
                    #mower_graphic.fill(red)
                    #cat_graphic.fill(black)
                    #gardener_graphic.fill(blue)
                    success = False
                    
            elif the_event.type == KEYUP:
                if the_event.key == K_LCTRL:
                    tidy_flag = False
        
        if incy > incmax:
            incy = incmax     
        elif incy < -incmax:
            incy = -incmax     
        if incx > incmax:
            incx = incmax
        elif incx < -incmax:
           incx = -incmax
           
        gardener_dead.after(100)

        if not game_over:                    
            if tidy_flag:
                lawn.items["gardener"].tidy(lawn)
                gardener_graphic.fill(white)
                incmax = 1
            else:
                gardener_graphic.fill(blue)
                incmax = 2            
            lawn.items["gardener"].rect.move_ip(incx,incy)
            
            if not lawn.rect.contains(lawn.items["gardener"].rect):
                oldx,oldy = -incx,-incy
                if lawn.items["gardener"].rect.top < 0:
                    incy = -incy - 1
                elif lawn.items["gardener"].rect.bottom > lawn.rect.bottom:
                    incy = -incy + 1
                    
                if lawn.items["gardener"].rect.left < 0:
                    incx = -incx - 1
                elif lawn.items["gardener"].rect.right > lawn.rect.right:
                    incx = -incx + 1
                    
                lawn.items["gardener"].rect.move_ip(oldx,oldy)
                
            mower_up.after(100)            
            mower_down.after(100)
            
            if incy !=0 or incx != 0:
                gardener_up.after(100)
                gardener_down.after(100)
                gardener_clean_up.after(100)            
                gardener_clean_down.after(100)
                
            cat_up = False
            catdx,catdy = catState.cat.dest
            if catdy < catState.cat.rect.top:
                cat_up = True
            
            
            if catState.state == "spawning" or catState.state == "walking":
                if cat_up:
                    cat_frame = cat_up_walk.next
                    cat_up_walk.after(100)
                else:   
                    cat_frame = cat_down_walk.next
                    cat_down_walk.after(100)
            elif catState.state == "running" or catState.state == "escaping":
                if cat_up:
                    cat_frame = cat_up_run.next
                    cat_up_run.after(100)
                else:   
                    cat_frame = cat_down_run.next
                    cat_down_run.after(100)
            elif catState.state == "crapping":
                cat_frame = cat_crap.next
                cat_crap.after(100)
                crapping = True
            elif catState.state == "scared":
                cat_frame = cat_scared
            
        lawn_graphic.fill(dark_green)
        for lane in lawn.mowed_rect:
            lawn_graphic.fill(green, lane)
        
        screen.blit(lawn_graphic,(0,0))
        
        if len(lawn.items["crap"]) != 0:
            i = 0
            for crap in lawn.items["crap"]:
                #crap_graphic = Surface((crap.width,crap.height))
                #crap_graphic.fill(brown)
                screen.blit(crap_graphic,crap.topleft)
                i = i + 1 # clunky
                
        #screen.blit(mower_graphic,lawn.items["mower"].rect.topleft)
        cat_graphic = cat_frames[cat_frame]
        screen.blit(cat_graphic,catState.cat.rect.topleft)
        
        #screen.blit(gardener_graphic,lawn.items["gardener"].rect.topleft)
        if lawn.items["gardener"].alive:
            if incy < 0:
                if tidy_flag:
                    screen.blit(gdnr_frames[gardener_clean_up.next],lawn.items["gardener"].rect.topleft)
                else:
                    screen.blit(gdnr_frames[gardener_up.next],lawn.items["gardener"].rect.topleft)
            else:
                if tidy_flag:
                    screen.blit(gdnr_frames[gardener_clean_down.next],lawn.items["gardener"].rect.topleft)
                else:
                    screen.blit(gdnr_frames[gardener_down.next],lawn.items["gardener"].rect.topleft)
        else:           
            screen.blit(gdnr_frames[gardener_dead.next],lawn.items["gardener"].rect.topleft)
        
        
        mx,my= lawn.items["mower"].rect.topleft
        if not lawn.items["mower"].coveredInCrap(lawn):            
            if mower_deltay < 0:
                screen.blit(mow_frames[mower_up.next],(mx+5,my))
            else:
                screen.blit(mow_frames[mower_down.next],(mx+5,my))
        else:
            if mower_deltay < 0:
                screen.blit(mow_frames[mower_up_crap_frame],(mx+5,my))
            else:
                screen.blit(mow_frames[mower_down_crap_frame],(mx+5,my))
            
        
        """
        text = scorefont.render(catState.state, True, (255,255,255), (0,0,0))
        screen.blit(text, (5,5))
        """
        
        if success:
            the_text = "well done!" 
            text = scorefont.render(the_text, True, (255,255,255), (0,0,0))
            screen.blit(text, (5,365))
            
            screen.blit(title_screen,(100,80))            
            the_text = "press 'r' to start"
            text = scorefont.render(the_text, True, (255,255,255), (0,0,0))
            screen.blit(text, (280,365))
        elif game_over:
            if not first_run:
                the_text = "bad luck!" 
                text = scorefont.render(the_text, True, (255,255,255), (0,0,0))
                screen.blit(text, (5,365))
            
            screen.blit(title_screen,(100,80))
            the_text = "press 'r' to start"
            text = scorefont.render(the_text, True, (255,255,255), (0,0,0))
            screen.blit(text, (150,285))
        """
        cat_text = "no_of_cats:" + "%d" % no_of_cats \
                    + " cat_posn:" + "%d" % catState.cat.rect.left \
                    + ",%d" % catState.cat.rect.top\
                    + " cat_dest:" + "%d,%d" % catState.cat.dest
        text = scorefont.render(cat_text, True, (255,255,255), (0,0,0))
        screen.blit(text, (5,385))
        """
    
        display.update()
    
        myClock.tick(100)
        display.get_surface().fill(0)
        
        if in_lane and not game_over:
            mower_sound_length = mower_sound_length - 1
            if mower_sound_length < 20:
                mixer.Sound.play(mower_sound)
                mower_sound_length = mower_sound.get_length() * 100
        else:
            mower_sound.stop()
            mower_sound_length = 19
        
        if not lawn.items["mower"].coveredInCrap(lawn) and not game_over:
            mower_deltay, in_lane, success = update_mower(lawn, mower_deltay, in_lane)
            if success:
                game_over = True
        elif not success:
            if not game_over:
                mixer.Sound.play(dead_mower)
            game_over = True
            mower_frame_count = mower_frame_count - 1
            if mower_frame_count < 0:
                mower_frame_count = 12
                if mower_crapped:
                    mower_graphic.fill(brown)
                else:
                    mower_graphic.fill(white)
                mower_crapped = not mower_crapped
        
        if not lawn.catOnLawn():
            catState.spawn(lawn)
            no_of_cats = no_of_cats + 1
            
        if lawn.items["gardener"].dead(lawn):
            if not game_over:
                mixer.Sound.play(dead_sound)
            game_over = True
            #gardener_graphic.fill(red)
        
        if not scared and lawn.scaresCat():
            mixer.Sound.play(scared_sound)
            
        scared = lawn.scaresCat()
        
        if not game_over:    
            catState.after(lawn, 100)
            if crapping and catState.state != "crapping":
                crapping = False
                mixer.Sound.play(crap_sound)
            
     
    # Be IDLE friendly
    quit ()

if __name__ == "__main__":
    main()